from svgwrite import cm, mm, px  

import tkinter as Tkinter
import tkinter.font as tkFont

import re

import utils.config as config

from utils.logger import LOGGER



class Text:
    
    _TK_ROOT = Tkinter.Tk() 
        
    def __init__(self, x=None, y=None, text=None, styleClass=None):

        self.__x = x + 10 # 10 = margin. TODO: handles margin with CSS
        self.__y = y + 10 # 10 = margin. TODO: handles margin with CSS
        self.__text = text
        self.__styleClass = styleClass
        
        self.__size = Text.getTextMetrics(self.__text, self.__styleClass)        
        print(self.__size)    
        
        self.__entityBlock = config.DWG.rect(insert=( (self.__x) * px, (self.__y) * px), 
                                            size=((self.__size['width']) * px, self.__size['height'] * px),
                                            stroke_width=0)        
        
        self.__entity = config.DWG.text(self.__text, 
                                        insert=( (self.__x) * px, 
                                        (self.__y + self.__size['height']*0.75) * px))
        self.__entity['class'] = "defaulttext " + self.__styleClass

    def addToGroup(self, group):
        group.add(self.__entityBlock)  
        group.add(self.__entity)
        


    @classmethod
    def __handleFontSize(cls, font_size):
        _font_size = re.sub(r'[^0-9]', '', font_size) 
        _font_units = re.sub(r'[^a-zA-Z]', '', font_size) 
        
        if(_font_units == 'px'):
            _font_size = 0 - int(_font_size)
            
        return _font_size        

    @classmethod
    def __handleFontWeight(cls, font_weight):
        if(font_weight is None or font_weight != 'bold'):
            return Tkinter.font.NORMAL
        return Tkinter.font.BOLD
    
    @classmethod    
    def __handleFontWeight(cls, font_weight):
        if(font_weight is None or font_weight != 'bold'):
            return Tkinter.font.NORMAL
        return Tkinter.font.BOLD    
    
    @classmethod
    def __handleFontStyle(cls, font_style):
        if(font_style is None or font_style != 'italic'):
            return Tkinter.font.ROMAN
        return Tkinter.font.ITALIC    
    
    @classmethod
    def __handleFontDecoration(cls, font_decoration):
        _font_decoration = {}
        _font_decoration['overstrike'] = 0
        _font_decoration['underline'] = 0     
        
        if(font_decoration is not None) :        
            if('line-through' in font_decoration.lower()):
                _font_decoration['overstrike'] = 1          
            if('underline' in font_decoration.lower()):
                _font_decoration['underline'] = 1      
        
        return _font_decoration    
    
    @classmethod
    def __getTextMetrics(cls, text, fontFamily, fontSize, fontStyle=None, fontWeight=None, fontDecoration=None):

        font = None
        
        _fontSize = Text.__handleFontSize(fontSize)
        _fontWeight = Text.__handleFontWeight(fontWeight)
        _fontStyle = Text.__handleFontStyle(fontStyle)
        _fontDecoration = Text.__handleFontDecoration(fontDecoration)
        
        """
        print("family= "+str(fontFamily)) 
        print("size= "+str(_fontSize))
        print("weight= "+str(_fontWeight)) 
        print("slant= "+str(_fontStyle))
        print("overstrike= "+str(_fontDecoration['overstrike']))
        print("underline= "+str(_fontDecoration['underline'] ))
        """
        
        font = tkFont.Font( family=fontFamily, 
                            size=_fontSize, 
                            weight=_fontWeight, 
                            slant=_fontStyle, 
                            overstrike=_fontDecoration['overstrike'],
                            underline=_fontDecoration['underline'] )
        
        if (font is not None) :
            _text_metrics = {}
            _text_metrics['width'] = font.measure(text)
            _text_metrics['height'] = font.metrics('linespace')
            return _text_metrics
        
        return None    
    
    @classmethod  
    def getTextMetrics(cls, text, styleClass):
        
        _textFontFamily = config.DEFAULTFONTFAMILY
        _textFontSize = config.DEFAULTFONTSIZE
        _textFontStyle = config.DEFAULTFONTSTYLE
        _textFontWeight = config.DEFAULTFONTWEIGHT 
        _textFontDecoration = config.DEFAULTFONTDECORATION       
        
        _textStyle = config.STYLESHEET['defaulttext']       
        try: 
            _textFontFamily = _textStyle['font-family']
        except:
            pass
        try: 
            _textFontSize = _textStyle['font-size']
        except:
            pass    
        try: 
            _textFontStyle = _textStyle['font-style']
        except:
            pass
        try: 
            _textFontWeight = _textStyle['font-weight']
        except:
            pass
        try: 
            _textFontDecoration = _textStyle['text-decoration']
        except:
            pass        
        
        _textStyle = config.STYLESHEET[styleClass]       
        try: 
            _textFontFamily = _textStyle['font-family']
        except:
            pass
        try: 
            _textFontSize = _textStyle['font-size']
        except:
            pass    
        try: 
            _textFontStyle = _textStyle['font-style']
        except:
            pass
        try: 
            _textFontWeight = _textStyle['font-weight']
        except:
            pass
        try: 
            _textFontDecoration = _textStyle['text-decoration']
        except:
            pass
        
        return Text.__getTextMetrics(text, _textFontFamily, _textFontSize, fontStyle=_textFontStyle, fontWeight=_textFontWeight, fontDecoration=_textFontDecoration)        
  


from svgwrite import cm, mm, px  

import utils.config as config

from utils.logger import LOGGER

from classes.Text import Text


class Block: 
        
    def __init__(self, x, y, width, height, name):
        self.__x = x
        self.__y = y
        self.__width = width
        self.__height = height
        self.__name = name
        
        self.__group = config.DWG.g(id=name, 
                                    stroke='green',
                                    stroke_width=1*px,
                                    fill='white'       )
        config.DWG.add(self.__group)
    
        self.__block = config.DWG.rect( insert=(x * px, y * px), 
                                        size=(width * px, height * px),
                                        rx = 10 * px,
                                        ry = 10 *px                      )
        self.__block['class'] = 'eteven'
    
        self.__group.add(self.__block)
    
        _text = Text(x, y, 'TITLÉ of thç blockuoeuepe', 'etname')
        _text.addToGroup(self.__group)


import logging


# Create a custom logger
def _buildLogger():

    _logger = logging.getLogger(__name__)

    logging.root.setLevel(logging.NOTSET)

    c_handler = logging.StreamHandler()
    f_handler = logging.FileHandler('./bsdm.log')
    c_handler.setLevel(logging.NOTSET)
    f_handler.setLevel(logging.ERROR)

    # Create formatters and add it to handlers
    c_format = logging.Formatter('%(levelname)s - %(message)s')
    f_format = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
    c_handler.setFormatter(c_format)
    f_handler.setFormatter(f_format)

    # Add handlers to the logger
    _logger.addHandler(c_handler)
    _logger.addHandler(f_handler)

    return _logger

LOGGER = _buildLogger()


# bsdm.py

import argparse

import svgwrite
from svgwrite import cm, mm, px  

from pathlib import Path

import utils.config as config

from utils.logger import LOGGER
from utils.StyleSheet import StyleSheet

from classes.Block import Block


def checkArgs():
    """
Checks and saves arguments passed when calling the program.

Returns the list of arguments if successful.
Otherwise, terminates the program.

Arguments: none
    """    
    
    parser = argparse.ArgumentParser(
        prog='BSDM',
        description='Beautiful SVG Data Model',
        epilog='Generate SVG beautiful diagram with CSS from Oracle Data Modeler.')

    parser.add_argument('-c','--css_file', required=False, help='css filename')
    parser.add_argument('data_model', help='Oracle Data Model directory')
    parser.add_argument('-o', '--output_svg_file', help='SVG output filename') 

    args = parser.parse_args()
    
    return args
    
    
def setOutputSVGPath(output_svg_file_arg):
    """
Builds, if necessary, the path for the program's output SVG file.

Returns a clean path corresponding to the one passed as argument.

Arguments:
output_svg_file_arg: a string designating the output SVG file.    
    """
    
    if(output_svg_file_arg):
        _svg_file_path = output_svg_file_arg
    else :
        _svg_file_path = "./data_model.svg"
    _svg_file_path = Path(_svg_file_path).resolve()
    _svg_file_path.parent.mkdir(parents=True, exist_ok=True)

    return _svg_file_path


def getInputCSS(input_css_file):
    """
Builds, if necessary, the path for the SVG CSS style file.

Returns a string containing the CSS style ot None if no CSS file passed in argument

Arguments:
input_css_file: a string designating the input CSS file.    
    """
    
    if(input_css_file):
        _css_file_path = input_css_file
    else :
        return None
    
    _css_file_path = Path(_css_file_path).resolve()

    if(_css_file_path.exists() and _css_file_path.is_file()):
        with open(str(_css_file_path), "r") as _css_file :
            return _css_file.read() 

    LOGGER.warning('Problem with CSS file. Skipping it')

    return None


def bsdm():

    args = checkArgs()
    
    LOGGER.info('BSDM is not what you think!')
    LOGGER.info(args.css_file + ", " +  args.output_svg_file + ", " +  args.data_model)    

    output_svg_path = setOutputSVGPath(args.output_svg_file)
    
    config.DWG = svgwrite.Drawing(str(output_svg_path), debug=True)
    
    cssStyleSheet = getInputCSS(args.css_file)    
    if(cssStyleSheet is not None) :
        config.DWG.embed_stylesheet(cssStyleSheet)
        StyleSheet.loadFromCSSStyleSheet(cssStyleSheet)        

    
    _block = Block(50, 50, 500, 300, 'first_block')
    
    config.DWG.save()


def main():
    bsdm()

if __name__ == "__main__":
    main()
    
    


import re
import json

import utils.config as config

from utils.logger import LOGGER

class StyleSheet: 
    
    __STYLESHEET = None

    @classmethod
    def __cleanElementString(cls, elementString):
        _elementString =  re.sub('/\*.*\*/', '', elementString) # remove comme (/* comment */ like)
        return _elementString
    
    @classmethod
    def __transformCSSToDict(cls, cssElements):
        
        _keysValues = re.compile(r"([a-zA-Z0-9(\);\s\#\"\-\.\_]+):([a-zA-Z0-9:\(\)\s\#\"\-\.\_]*)")
    
        _jsonElementItems= {}
            
        for _keyValue in _keysValues.finditer(cssElements):
            _jsonElementItems[_keyValue.group(1).strip()] = _keyValue.group(2).strip()
        
        return(_jsonElementItems)
    
    @classmethod
    def __substituteVar(cls, cssStyleSheet, styleSheetVar):

        _cssStyleSheet = cssStyleSheet
        
        for var  in styleSheetVar:
            _cssStyleSheet = re.sub('var\s*\(\s*\-\-'+var+'\s*\)', styleSheetVar[var], _cssStyleSheet)
        
        return _cssStyleSheet
    
    @classmethod
    def __loadElementsStyle(cls, cssStyleSheet, styleSheetVar):
        
        _cssStyleSheet = re.sub('/\*.*\*/', '',  cssStyleSheet)                            # remove comme (/* comment */ like)        
        _cssStyleSheet = StyleSheet.__substituteVar(_cssStyleSheet, styleSheetVar)
        
        _elementPattern = re.compile(r"\.([a-zA-Z0-9:\(\);\s\#\"\-\.\_]+)[\s]*{\s*([a-zA-Z0-9(\);\s\#\"\-\.\_]+:[a-zA-Z0-9:\(\);\s\#\"\-\.\_]*)}")
        
        _elementStyle = {}
        
        for _element in _elementPattern.finditer(_cssStyleSheet):
            _elementStyle[_element.group(1).strip()] = StyleSheet.__transformCSSToDict(_element.group(2))
        
        return _elementStyle

    @classmethod
    def __listVars(cls, cssStyleSheet):
        _styleSheetVars = {}

        _cssStyleSheet = StyleSheet.__cleanElementString(cssStyleSheet)
        
        _varPattern = re.compile(r"\-\-([a-zA-Z0-9\s\-]+:[a-zA-Z0-9\s\#\"\-]+);")

        for _var in _varPattern.finditer(_cssStyleSheet):
            tmp = _var.group(1).split(":")
            _styleSheetVars[tmp[0].strip()] = tmp[1].replace('"','').strip()
            
        return _styleSheetVars

    @classmethod
    def loadFromCSSStyleSheet(cls, cssStyleSheet):
        _styleSheetVars = StyleSheet.__listVars(cssStyleSheet)    
        StyleSheet.__STYLESHEET = StyleSheet.__loadElementsStyle(cssStyleSheet, _styleSheetVars) 
        config.STYLESHEET = StyleSheet.__STYLESHEET